<?php

namespace GamesShop\Routing\Api\Web;

use Doctrine\Common\Collections\Criteria;
use Doctrine\ORM\EntityManager;
use GamesShop\ContainerHandler;
use GamesShop\Entities\Games\Game;
use GamesShop\Entities\Games\Key;
use GamesShop\Entities\Games\KeyState;
use GamesShop\Login\LoginHandler;
use GamesShop\Login\UserPermission;
use GamesShop\UserManager;
use Laminas\Diactoros\Response\JsonResponse;
use League\Route\Http\Exception\ForbiddenException;
use League\Route\Http\Exception\NotFoundException;
use League\Route\Http\Exception\UnauthorizedException;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;

class GetGameData
{

    public function __construct(
        private readonly LoginHandler $loginHandler,
        private readonly EntityManager $entityManager,
    )
    {
    }

    public function __invoke(ServerRequestInterface $request): ResponseInterface {

        if (!$this->loginHandler->isLoggedIn()) {
            throw new UnauthorizedException();
        }

        $user = $this->loginHandler->getCurrentUser();
        if (!$user->getPermission()->hasLevel(UserPermission::VIEWER)) {
            throw new ForbiddenException();
        }

        $game = $this->entityManager->getRepository(Game::class)
            ->find($request->getQueryParams()['gameid']);
        if (!$game) {
            throw new NotFoundException();
        }
        
        $applicableLists = ContainerHandler::get(UserManager::class)
            ->getApplicableGameLists($user);
        
        $keyRepo = $this->entityManager->getRepository(Key::class);
        $keys = $keyRepo->matching(Criteria::create()
            ->where(Criteria::expr()->in('list', $applicableLists->toArray()))
            ->andWhere(Criteria::expr()->eq('game', $game))
            ->andWhere(Criteria::expr()->eq('state', KeyState::AVAILABLE))
        );
        
        return new JsonResponse([
            'name' => $game->getName(),
            'id' => $game->getId(),
            'keys' => $keys->map(fn (Key $key) => [
                'id' => $key->getId(),
                'store' => [
                    'name' => $key->getStore()->getName(),
                    'icon' => $key->getStore()->getIcon()
                ],
                'fromWhere' => $key->getFromWhere(),
                'list' => [
                    'owner' => $key->getList()->getOwner()->getName(),
                    'name' => $key->getList()->getName(),
                ],
            ])->toArray(),
        ]);
    }
}